{ Demonstration program for ChroMathsome - Statistics.

  This demo program shows how to use statvectors and statmatrices.
  As an example, we'll show the matrix- and vector capabilities of
  Chromathsome - Statistics in a market with challenging economic issues.

  The same analytical tools and techniques can be applied in many other industries.

  The reader may do with this code whatever he/she wants.
  Author: John Merrit (john.merrit@net4all.be)
  Release date: May '98

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  ChroMathsome - Statistics 1996-98 John Merrit. All Rights Reserved

  Brought to you by the letter "S".             }
unit harvest;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, Statistc;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    Close1: TMenuItem;
    Harvest1: TMenuItem;
    Crop1: TMenuItem;
    OptimalYield1: TMenuItem;
    procedure Close1Click(Sender: TObject);
    procedure Crop1Click(Sender: TObject);
    procedure OptimalYield1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Close1Click(Sender: TObject);
begin
     Close;
end;

{ A harvester has a crop of 120 silver fir trees to sell annually for Christmas trees.
  After last year's harvest, his forest had the following configuration:
  Class   Height Interval (in feet)  Number of Trees
    1     [ 0 , 4 )                         15
    2     [ 4 , 8 )                         20
    3     [ 8 ,12 )                         35
    4     [12 ,16 )                         30
    5     [16 ,inf)                         20
  During the growth period, six trees in class 1 grew to the next height class, as did
  thirteen trees in class 2, ten trees in class 3 and four trees in class 4.
  If the harvester sustainably harvests eight trees of class 2, six trees of class 3, thirteen
  trees of class 4 and six trees of class 5, what is the configuration of his crop after
  harvest and replanting?                                                                  }
procedure TForm1.Crop1Click(Sender: TObject);
var Nonharvest, Harvest: StatVector;
    Growth, Replacement, Crop, Temp1Matrix, Temp2Matrix: StatMatrix;
    i: integer;
begin
     InitStatVector(Nonharvest, 5);
     InitStatVector(Harvest, 5);

     InitStatMatrix(Growth, 5, 5);
     InitStatMatrix(Replacement, 5, 5);
     InitStatMatrix(Crop, 1, 1);
     InitStatMatrix(Temp1Matrix, 1, 1);
     InitStatMatrix(Temp2Matrix, 1, 1);

     try
     { Populate nonharvest vector }
     Nonharvest.Vector^[1]:= 15;
     Nonharvest.Vector^[2]:= 20;
     Nonharvest.Vector^[3]:= 35;
     Nonharvest.Vector^[4]:= 30;
     Nonharvest.Vector^[5]:= 20;

     { Calculate growth fractions for each height class -> Growth matrix }
     { Tree growth between harvests is designated by gi, the fraction of trees that
       grow from class i to class i + 1.
       1 - gi = fraction of trees that remain in class i.
       Growth matrix =
       [[ 1-g1 0 0 ... 0]
        [g1 1-g2 0 ... 0]
        [0 g2 1-g3 ... 0]
        [ ... ... ... . ]
        [...    1-gn-1 0]
        [0 0 ...  gn-1 1]]
        }

       Growth.Matrix^[1]^[1]:= 1 - 6/15;
       Growth.Matrix^[2]^[1]:= 6/15;
       Growth.Matrix^[2]^[2]:= 1 - 13/20;
       Growth.Matrix^[3]^[2]:= 13/20;
       Growth.Matrix^[3]^[3]:= 1 - 10/35;
       Growth.Matrix^[4]^[3]:= 10/35;
       Growth.Matrix^[4]^[4]:= 1 - 4/30;
       Growth.Matrix^[5]^[4]:= 4/30;
       Growth.Matrix^[5]^[5]:= 1;
       { All other elements of matrix Growth must be 0 but since InitStatMatrix initializes
         the matrix as a nil matrix ... }

      { Populate vector Harvest }
      Harvest.Vector^[1]:= 0;
      Harvest.Vector^[2]:= 8;
      Harvest.Vector^[3]:= 6;
      Harvest.Vector^[4]:= 13;
      Harvest.Vector^[5]:= 6;

      { Populate replacement matrix:
        [[1 1 1 ... 1]
         [0 0 0 ... 0]
         [... ... ...]
         [0 0 0 ... 0]]
         }
      for i:= 1 to Replacement.Col do
      Replacement.Matrix^[1]^[i]:= 1;

      { Now, we're going to compute the configuration of the forest after
        harvest.
        Forest after harvest = Forest ready for harvest - harvest + new seedlings.
        Growth * Nonharvest - Harvest + Replacement * Harvest = Forest after harvest. }
     { 1. Growth * Nonharvest }
     StatVectorToStatMatrix(Temp1Matrix, Nonharvest); // Temp1Matrix = 1 * 5 matrix ...
     StatMatrixTranspose(Temp1Matrix); // ... make it a 5 * 1 matrix
     StatMatrixMultiplication(Crop, Growth, Temp1Matrix); // Growth * Nonharvest = Crop

     { 2. Replacement * Harvest }
     StatVectorToStatMatrix(Temp1Matrix, Harvest); // Temp1Matrix = 1 * 5 matrix ...
     StatMatrixTranspose(Temp1Matrix); // ... make it a 5 * 1 matrix
     StatMatrixMultiplication(Temp2Matrix, Replacement, Temp1Matrix); // Replacement * Harvest = Temp2Matrix

     StatVectorToStatMatrix(Temp1Matrix, Harvest); // Temp1Matrix = 1 * 5 matrix ...
     StatMatrixTranspose(Temp1Matrix); // ... make it a 5 * 1 matrix

     { 3. Growth * Nonharvest (= Crop) - Harvest( = Temp1Matrix) }
     StatMatrixSubtraction(Crop, Crop, Temp1Matrix);

     { 4. Add Replacement * Harvest ( = Temp2Matrix) }
     StatMatrixAddition(Crop, Crop, Temp2Matrix);

     { Final result.
       Transform statmatrix Crop to a statvector (= new nonharvest vector)
       via StatMatrixToStatVector(...);                                                 }
     StatMatrixToStatVector(Nonharvest, Crop);
     for i:= 1 to Nonharvest.Dimension do
     begin
     Memo1.Lines.Add('Component ' + IntToStr(i) + ' of nonharvest vector: ' + FloatToStr(Nonharvest.Vector^[i]));
     end;

     finally
     DestroyStatVector(Nonharvest);
     DestroyStatVector(Harvest);

     DestroyStatmatrix(Growth);
     DestroyStatmatrix(Replacement);
     DestroyStatmatrix(Crop);
     DestroyStatmatrix(Temp1Matrix);
     DestroyStatmatrix(Temp2Matrix);
     end;
end;

{ If the harvester wishes to optimize his profit year after year, he must determine the optimal
  sustainable yield.
  This is achieved by harvesting all of the trees from one particular height/price class and
  no trees from any other class.
  The sustainable yield is thus a function of both price and growth rate, but independent of the
  current nonharvest vector.
  Note that if class k provides the maximum yield, the first year all classes >= k are
  harvested.
  In the following years only class k is harvested, and no trees will ever be present in
  higher classes.
  Suppose the market prices for the five classes are
  p1 = $0, p2 = $50, p3 = $100, p4 = $150 and p5 = $200.
  Determine which height class should be harvested.
  Note: this uses the data from Crop, above.                  }
procedure TForm1.OptimalYield1Click(Sender: TObject);
var Yield: StatVector;
    GrowthRatio, Price, Temp1Matrix: StatMatrix;
    i: integer;
    Value: extended;
begin
     InitStatVector(Yield, 5);

     InitStatMatrix(GrowthRatio, 5, 1);
     InitStatMatrix(Price, 5, 5);
     InitStatMatrix(Temp1Matrix, 1, 1);

     try
     { Populate price matrix.
       [[p1 0 ... 0]
        [0 p2 ... 0]
        [... ... ..]
        [0 0 ... pn]] }
     Price.Matrix^[1]^[1]:= 0;
     Price.Matrix^[2]^[2]:= 50;
     Price.Matrix^[3]^[3]:= 100;
     Price.Matrix^[4]^[4]:= 150;
     Price.Matrix^[5]^[5]:= 200;
     { All other elements of matrix Price must be 0 but since InitStatMatrix initializes
         the matrix as a nil matrix ... }

     { Compute growth ratio matrix:
       [[gg1]
        [gg2]
        [...]
        [ggn]]
        where gg1 = 0 and
        ggi = 1/(Sum(1/gk)); Sum = summation with index k from k = 1 to i-1.
        Note: gi are defined in previous procedure Crop.  }
     GrowthRatio.Matrix^[1]^[1]:= 0;
     GrowthRatio.Matrix^[2]^[1]:= 1/(15/6);
     GrowthRatio.Matrix^[3]^[1]:= 1/(15/6 + 20/13);
     GrowthRatio.Matrix^[4]^[1]:= 1/(15/6 + 20/13 + 35/10);
     GrowthRatio.Matrix^[5]^[1]:= 1/(15/6 + 20/13 + 35/10 + 30/4);

     { Find yield vector:
       Yield = Price * GrowtRatio * S
       with S = total number of trees = 120 (see crop, higher) }
     StatMatrixMultiplication(Temp1Matrix, Price, GrowthRatio);
     StatMatrixScalarMultiplication(Temp1Matrix, 120);
     { Convert Temp1Matrix to vector Yield. }
     StatMatrixToStatVector(Yield, Temp1Matrix);
     for i:= 1 to Yield.Dimension do
     begin
     Memo1.Lines.Add('Component ' + IntToStr(i) + ' of yield vector: ' + FloatToStr(Yield.Vector^[i]));
     end;
     { Find largets number in Yield = highest income. }
     MaxAbsValueInStatVector(Value, i, Yield);
     Memo1.Lines.Add('Class ' + IntToStr(i) + ' should be harvested to maximize the annual sustainable yield.');

     finally
     DestroyStatVector(Yield);

     DestroyStatmatrix(GrowthRatio);
     DestroyStatmatrix(Price);
     DestroyStatmatrix(Temp1Matrix);
     end;
end;

end.
